/* BotServ functions.
 *
 * Auspice Services is copyright (c) 2000-2001 In Mean.
 *     E-mail: <inmean@pleaselook.com>
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "../inc/services.h"
#include "../inc/timeout.h"
#include "bs-help.c"

static int nhostline = 0;
static int hostline_size = 0;
static struct s_hline {
	char *nick;
	char *host;
	char who[NICKMAX];
	long reserved[4];
} *s_hlines = NULL;

static void do_iihelp(const char *source);
void load_hline_db(void);
void save_hline_db(void);
void do_hostline(const char *nick);
int del_hostline(const char *nick);
void do_adlhost(const char *source);
static void do_blang(const char *source);

void botserv(const char *source, char *buf)
{
    char *cmd;
    char *s, *chan, *nick, *host;
    int i;
    User *u = finduser(source);

    cmd = strtok(buf, " ");

    if (!cmd) {
        return;
    } else if (stricmp(cmd, "HELP") == 0) {
        do_iihelp(source);
    } else if (stricmp(cmd, "HOSTSERV") == 0) {
        do_adlhost(source);
    } else if (stricmp(cmd, "LMODE") == 0) {
        do_blang(source);
    } else if (stricmp(cmd, "\1PING") == 0) {
        if (!(s = strtok(NULL, "")))
            s = "\1";
        notice(s_OperServ, source, "\1PING %s", s);

    } else {
        notice(s_BotServ, source,
                "Unrecognized command \2%s\2.", cmd);
    }
}

/*************************************************************************/

/* HELP command. */

static void do_iihelp(const char *source)
{
    const char *cmd = strtok(NULL, " ");
    if (!cmd)
        notice_list(s_BotServ, source, bs_help);
    else if (stricmp(cmd, "HOSTSERV") == 0)
        notice_list(s_BotServ, source, hostserv_help);
    else
        notice(s_BotServ, source,
			"No help available for command \2%s\2.", cmd);
    notice(s_BotServ, source, "\2END OF HELP\2");
}
/*************************************************************************/

void load_hline_db(void)
{
    FILE *f;
    int i;

    if (!(f = open_db(s_BotServ, HLINE_DB, "r")))
        return;

    switch (i = get_file_version(f, HLINE_DB)) {
      case 5:
        nhostline = fgetc(f) * 256 + fgetc(f);
        if (nhostline < 8)
            hostline_size = 16;
        else
            hostline_size = 2*nhostline;
        s_hlines = smalloc(sizeof(*s_hlines) * hostline_size);
        if (!nhostline) {
            fclose(f);
            return;
        }
        if (nhostline != fread(s_hlines, sizeof(*s_hlines), nhostline, f))
            fatal_perror("Read error on %s", HLINE_DB);
        for (i = 0; i < nhostline; i++) {
            s_hlines[i].nick = read_string(f, HLINE_DB);
            s_hlines[i].host = read_string(f, HLINE_DB);
        }
        break;
      default:
        fatal("Unsupported version (%d) on %s", i, HLINE_DB);
    } /* switch (version) */

    close_db(f, HLINE_DB);
}
/***********************************************************/

void save_hline_db(void)
{
    FILE *f;
    int i, count = 0;

    f = open_db("HOSTSERV", HLINE_DB, "w");
    fputc(nhostline/256, f);
    fputc(nhostline & 255, f);
    if (fwrite(s_hlines, sizeof(*s_hlines), nhostline, f) != nhostline)
        fatal_perror("Write error on %s", HLINE_DB);
    for (i = 0; i < nhostline; i++) {
        write_string(s_hlines[i].nick, f, HLINE_DB);
        write_string(s_hlines[i].host, f, HLINE_DB);
    }
    close_db(f, HLINE_DB);

}

/**********************************************************/

int is_hostline(const char *nick)
{
    NickInfo *ni;
    int i;

    for (i = 0; i < nhostline; i++) {
        if (stricmp(s_hlines[i].nick,nick) == 0) {
            return 1;
        }
    }
    return 0;
}

/*********************************************************/
void do_hostline(const char *nick)
{
    NickInfo *ni;
    int i;
    for (i = 0; i < nhostline; i++) {
        if (stricmp(s_hlines[i].nick,nick) == 0) {
	    send_cmd(s_BotServ,"CHGHOST %s %s",nick,s_hlines[i].host);
	    return;
        }
    }
    return;
}
/**********************************************************/
int del_hostline(const char *nick)
{
    int i;

    for (i = 0; i < nhostline && stricmp(s_hlines[i].nick, nick) != 0; i++);
    if (i < nhostline) {
        free(s_hlines[i].nick);
        free(s_hlines[i].host);
        nhostline--;
        if (i < nhostline)
        bcopy(s_hlines+i+1, s_hlines+i, sizeof(*s_hlines) * (nhostline-i));
        return 1;
    } else {
        return 0;
    }
}

/***********************************************************/
void do_adlhost(const char *source)
{
    char *cmd = strtok(NULL," ");
    char *nick, *host, *info;
    NickInfo *ni;
    int i;
    char *s;

    if (!cmd) {
        goto err;
    }

    if (stricmp(cmd, "LIST") == 0) {
        s = strtok(NULL, " ");
        if (!s)
            s = "*";

        notice(s_BotServ, source, "\2Current HostServ list:\2");
        for (i = 0; i < nhostline; i++) {
            if (!s || match_wild(s, s_hlines[i].nick)) {

#ifndef SKELETON
              if (ni = findnick(s_hlines[i].nick))
                notice(s_BotServ, source, " %d) %-16s (%s)",
                                i+1, s_hlines[i].nick, s_hlines[i].host);
              else
                notice(s_BotServ, source, " %d) %-16s (Not Registered)",
                                i+1, s_hlines[i].nick);

#else
            notice(s_BotServ, source, " %d) %s-16s (N/A in SKELETON version)",
                    i+1, s_hlines[i].nick);
#endif
            }
        }
        notice(s_BotServ,source,"\2End of List\2");


    } else if (!is_services_oper(source)) {
        notice(s_BotServ, source, "Permission denied.");
        return;

    } else if (stricmp(cmd,"ADD") == 0) {
        nick = strtok(NULL, " ");
        host = strtok(NULL, "");
        if (!nick || !host) {
            notice(s_BotServ, source, "Syntax: \2HOSTSERV ADD\2 <nick> <host>");
            return;
        }
        if (!findnick(nick)) {
                notice(s_OperServ, source, "Nick %s isn't registered!", nick);
                return;
        }

        for (i = 0; i < nhostline; i++) {
            if (stricmp(s_hlines[i].nick,nick) == 0) {
                notice(s_BotServ,source,"%s is already present "
                    "on the HostServ list.", nick);
                return;
            }
        }

        if (nhostline >= hostline_size) {
            if (hostline_size < 8)
                hostline_size = 8;
            else
                hostline_size *= 2;
            s_hlines = srealloc(s_hlines, sizeof(*s_hlines) *hostline_size);
        }

        if (!nhostline || (nhostline < MAX_HOSTSERV) || (MAX_HOSTSERV == 0)) {
            s_hlines[nhostline].nick = sstrdup(nick);
            s_hlines[nhostline].host = sstrdup(host);
            strscpy(s_hlines[nhostline].who, source, NICKMAX);

            notice(s_BotServ, source, "Set nick %s to HostServ", nick);
            wallops(s_BotServ, "Auto CHGHOST nick %s by request of %s to: %s",
                    nick, source, host);
            ++nhostline;
            if (readonly) {
                notice(s_BotServ, source,
                    "\2Notice:\2 Changes will not be saved! "
                    " Services is in read-only mode.");
            }
        }
        else {
            notice(s_BotServ,source,"Sorry, HOSTSERV list is full.");
            wallops(SERVER_NAME,"\2Warning\2: HOSTSERV(nick) list is full. (%d)", MAX_HOSTSERV);
        }

    } else if (stricmp(cmd, "INFO") == 0) {
        int i;
        nick = strtok(NULL, " ");
        info = strtok(NULL, "");

        if (!is_services_root(source)) {
            notice(s_BotServ, source, "Permission denied.");
            return;
        }

        if ((nick == NULL) || (info == NULL))
          return;

            if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= nhostline) {

                free(s_hlines[i-1].host);
                s_hlines[i-1].host = sstrdup(info);
                notice(s_BotServ, source, "CHGHOST host for %s changed to: %s",
                             s_hlines[i-1].nick, sstrdup(info));
                return;
            }

         for (i = 0; i < nhostline; i++) {
               if (stricmp(s_hlines[i].nick, nick) == 0) {

                   free(s_hlines[i].host);
                   s_hlines[i].host = sstrdup(info);
                   notice(s_BotServ, source, "CHGHOST host for %s changed to: %s",
                               s_hlines[i].nick, s_hlines[i].host);
               }
         }

    } else if (stricmp(cmd, "DEL") == 0) {
        int i;
        if (nick = strtok(NULL, " ")) {
            if (strspn(nick, "1234567890") == strlen(nick) &&
                                (i = atoi(nick)) > 0 && i <= nhostline) {
                strcpy(nick,s_hlines[i-1].nick);
            }
            if (del_hostline(nick)) {
                notice(s_BotServ, source, "%s removed from HostServ list.", nick);
                wallops(s_BotServ,
                        "%s removed %s from the HostServ list.",
                        source, nick);
                if (readonly) {
                    notice(s_BotServ, source,
                        "\2Notice:\2 Changes will not be saved!  Services is in readonly mode.");
                }
            } else {
                notice(s_BotServ, source, "%s not found on HostServ list.",
                    nick);
            }
        } else {
            notice(s_BotServ, source, "Syntax: \2HOSTSERV DEL\2 <nick>");
        }
    } else if (stricmp(cmd, "LIST") == 0) {
        s = strtok(NULL, " ");
        if (!s)
            s = "*";

        notice(s_BotServ, source, "\2Current HOSTSERV list:\2");
        for (i = 0; i < nhostline; i++) {
            if (!s || match_wild(s, s_hlines[i].nick)) {
#ifndef SKELETON
              if (ni = findnick(s_hlines[i].nick)) {
                notice(s_BotServ, source, " %d) %-16s (%s)",
                                i+1, s_hlines[i].nick, ni->last_usermask);
              }
#else
            notice(s_BotServ, source, " %d) %s-16s (N/A in SKELETON version)",
                    i+1, s_hlines[i].nick);
#endif
            }
        }
        notice(s_BotServ,source,"\2End of List\2");
    } else {
err:
        notice(s_BotServ,source,"Syntax: \2HOSTSERV\2 [ADD|DEL|LIST] [<nick>] [<host>]");
        return;
    }
}

/*************************************************************************/
static void do_blang(const char *source)
{
    const char *cmd = strtok(NULL, " ");
    if (!cmd)
        notice(s_BotServ,source,"Syntax: \2LMODE\2 <Country Domain> ex: KH MY EN FR...");
    else if (stricmp(cmd, "KH") == 0)
	send_cmd(s_BotServ,"SWHOIS %s :is prefer to speak Khmer language.",source);
    else if (stricmp(cmd, "MY") == 0)
	send_cmd(s_BotServ,"SWHOIS %s :is prefer to speak Malay language.",source);
    else if (stricmp(cmd, "EN") == 0)
	send_cmd(s_BotServ,"SWHOIS %s :is prefer to speak English language.",source);
    else if (stricmp(cmd, "FR") == 0)
	send_cmd(s_BotServ,"SWHOIS %s :is prefer to speak French language.",source);
    else
        notice(s_BotServ, source, "There is no \2%s\2 language support yet.", cmd);
}
/***********************************************************************/
