/*
 * Auspice Services is copyright (c) 2000-2001 In Mean
 *     E-mail: auspice@pleaselook.com
 *
 * SirvNET Services is copyright (c) 1998-2000 Trevor Klingbeil.
 *     E-mail: <priority1@dal.net>
 * Originally based on EsperNet Services(c) by Andy Church.
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "../inc/services.h"

static Channel *chanlist = NULL;
static char *slist[6];

static void get_sjoin_list(char *list);
static void extract_nicks();

/*************************************************************************/

/* Bahamut (TS3) SJOIN procedure */

void do_sjoin(char *source, int ac, char **av)
{

   Channel *chan;
   char *schan = av[2];
   char *s, *who;
   char *mv[3], *rchar;
   int add = 1, args = 0, start = 4, i, x=0;
   int newchan = 0;
   struct c_userlist *u; 
   struct u_chanlist *c;
   User *user;   

   chan = findchan(av[2]);
   if (!chan) {

      if (debug)
           log("Creating new channel %s via sjoin", av[2]);

      chan = scalloc(sizeof(Channel), 1);
      chan->next = chanlist;
      if (chanlist)
          chanlist->prev = chan;
      chanlist = chan;
      strscpy(chan->name, av[2], sizeof(chan->name));
      chan->creation_time = time(NULL);
      newchan = 1;


   }

/*   We can't use source here, returns server name now


   if (debug)
     log("debug: %s joins %s", source, schan);


*/


   s = av[3];

   while (*s) {
        switch(*s++) {

            case 'i':
                chan->mode |= CMODE_I;

                break;

            case 'm':
                chan->mode |= CMODE_M;
                break;

            case 'r':
                chan->mode |= CMODE_r;  /* Registered Channel */
                break;

            case 'n':
                chan->mode |= CMODE_N;
                break;

            case 'p':
                chan->mode |= CMODE_P;
                break;

            case 's':
                chan->mode |= CMODE_S;
                break;

            case 't':
                chan->mode |= CMODE_T;
                break;

            case 'c':
                chan->mode |= CMODE_C;
                break;

            case 'Q':
                chan->mode |= CMODE_Q;
                break;

/*            case 'R':
                chan->mode |= CMODE_RN;
                break;
*/
            case 'l':
                if (chan->limit > 0)
                   chan->limit = 0;
                chan->limit = atoi(av[4]);

                break;



            case 'k':
                if (chan->key) {
                   free(chan->key);
                   chan->key = NULL;
                }
                if (chan->limit)
                     chan->key = sstrdup(av[5]);
                else
                     chan->key = sstrdup(av[4]);
                args++;
                break;

        }
    
   }         


#ifndef SKELETON
     check_modes(chan->name);
#endif


   if (stricmp(av[3], "+") != 0) {

       if (chan->key && (chan->limit > 0))
           start += 2;    
       else if ((!chan->key && (chan->limit > 0)) || 
                 (chan->key && (chan->limit == 0)))
           start++;
   }

   mv[0] = sstrdup(chan->name);




  get_sjoin_list(av[start]);


  for (x = 0; slist[x]; x++) {

         if (strchr(slist[x], '@') && strchr(slist[x], '+') && strchr(slist[x], '%')) {
             mv[2] = strtok(slist[x], "@+%");
             mv[1] = sstrdup("+ovh");
             do_cmode(source, 3, mv);
             
         } else if (strchr(slist[x], '@')) {
             mv[2] = strtok(slist[x], "@");
             mv[1] = sstrdup("+o");
             do_cmode(source, 3, mv);

         } else if (strchr(slist[x], '+')) {
             mv[2] = strtok(slist[x], "+");
             mv[1] = sstrdup("+v");
             do_cmode(source, 3, mv);

         } else if (strchr(slist[x], '%')) {
             mv[2] = strtok(slist[x], "%");
             mv[1] = sstrdup("+h");
             do_cmode(source, 3, mv);

         } else
             mv[2] = sstrdup(slist[x]);


      slist[x] = sstrdup(mv[2]);
      user = finduser(mv[2]);


#ifndef SKELETON
      if (check_kick(user, chan->name))
         continue;
#endif


      if (!is_chanop(mv[2], schan))
           chan_adduser(user, schan);




      u = smalloc(sizeof(struct c_userlist));
      u->next = chan->users;
      u->prev = NULL;
      if (chan->users)
         chan->users->prev = u;
      chan->users = u;
      u->user = user;

      c = smalloc(sizeof(*c));
      c->next = user->chans;
      c->prev = NULL;
      if (user->chans)
          user->chans->prev = c;
      user->chans = c;
      c->chan = findchan(chan->name);        


/* do_join removed - as I said, I got around to improving it :P */


#ifndef SKELETON
      if (u && newchan && (x < 1)) /* x to prevent repeated topics */
            restore_topic(chan->name);
#endif
   }

   mv[1] = NULL; mv[2] = NULL; 
   i++;    


   for (x = 0; slist[x]; x++) {
#ifndef SKELETON
       user = finduser(slist[x]);
       if (!rec_services_root(user->nick))
           check_invite(user, chan->name);
#endif
       slist[x] = NULL;
   }
}

/*************************************************************************/

static void get_sjoin_list(char *list)
{
    slist[0] = strtok(list, " ");

    extract_nicks();
}


/*************************************************************************/

static void extract_nicks()
{
   char *tmp;
   int i = 1;

   while ((tmp = strtok(NULL, " ")) != NULL)
   {
       slist[i++] = tmp;

   }
   
}


/*************************************************************************/



/* Return statistics.  Pointers are assumed to be valid. */

void get_channel_stats(long *nrec, long *memuse)
{
    long count = 0, mem = 0;
    Channel *chan;
    struct c_userlist *cu;
    int i;

    for (chan = chanlist; chan; chan = chan->next) {
	count++;
	mem += sizeof(*chan);
	if (chan->topic)
	    mem += strlen(chan->topic)+1;
	if (chan->key)
	    mem += strlen(chan->key)+1;
	mem += sizeof(char *) * chan->bansize;
	for (i = 0; i < chan->bancount; i++) {
	    if (chan->bans[i])
		mem += strlen(chan->bans[i])+1;
	}
	for (cu = chan->users; cu; cu = cu->next)
	    mem += sizeof(*cu);
	for (cu = chan->chanops; cu; cu = cu->next)
	    mem += sizeof(*cu);
	for (cu = chan->voices; cu; cu = cu->next)
	    mem += sizeof(*cu);
    }
    *nrec = count;
    *memuse = mem;
}

/*************************************************************************/

/* Return the Channel structure corresponding to the named channel, or NULL
 * if the channel was not found. */

Channel *findchan(const char *chan)
{
    Channel *c = chanlist;

    while (c) {
	if (stricmp(c->name, chan) == 0)
	    return c;
	c = c->next;
    }
    return NULL;
}

/*************************************************************************/

/* Add/remove a user to/from a channel, creating or deleting the channel as
 * necessary. */

void chan_adduser(User *user, const char *chan)
{
    Channel *c = findchan(chan);
    int newchan = !c;
    struct c_userlist *u;

#ifndef BAHAMUT
    if (newchan) {
	if (debug)
	    log("debug: Creating channel %s", chan);
	/* Allocate pre-cleared memory */
	c = scalloc(sizeof(Channel), 1);
	c->next = chanlist;
	if (chanlist)
	    chanlist->prev = c;
	chanlist = c;
	strscpy(c->name, chan, sizeof(c->name));
	c->creation_time = time(NULL);

#ifndef SKELETON
	/* This will restore locked modes and saved topic */

        check_invite(user, chan);
	check_modes(chan);
        restore_topic(chan);

#endif
    }
#endif  /* bahamut */



#ifndef SKELETON    
    if (check_should_voice(user, chan)) {
	u = smalloc(sizeof(struct c_userlist));
	u->next = c->voices;
	u->prev = NULL;
	if (c->voices)
	    c->voices->prev = u;
	c->voices = u;
	u->user = user;
    }

    if (check_should_halfop(user, chan)) {
	u = smalloc(sizeof(struct c_userlist));
	u->next = c->halfop;
	u->prev = NULL;
	if (c->halfop)
	    c->halfop->prev = u;
	c->halfop = u;
	u->user = user;
    }

    if (!check_opguard(user, chan) || newchan)
        ;

    else if (check_should_op(user, chan)) {
	u = smalloc(sizeof(struct c_userlist));
	u->next = c->chanops;
	u->prev = NULL;
	if (c->chanops)
	    c->chanops->prev = u;
	c->chanops = u;
	u->user = user;
}

#endif

#ifndef BAHAMUT
    u = smalloc(sizeof(struct c_userlist));
    u->next = c->users;
    u->prev = NULL;
    if (c->users)
	c->users->prev = u;
    c->users = u;
    u->user = user;
#endif

}

void chan_deluser(User *user, Channel *c)
{
    struct c_userlist *u;
    int i;


    for (u = c->users; u && u->user != user; u = u->next)
	;
    if (!u)
	return;
    if (u->next)
	u->next->prev = u->prev;
    if (u->prev)
	u->prev->next = u->next;
    else
	c->users = u->next;
    free(u);


    for (u = c->chanops; u && u->user != user; u = u->next)
    ;
    if (u) {
	if (u->next)
	    u->next->prev = u->prev;
	if (u->prev)
	    u->prev->next = u->next;
	else
	    c->chanops = u->next;
	free(u);
    }

    for (u = c->voices; u && u->user != user; u = u->next)
	;
    if (u) {
	if (u->next)
	    u->next->prev = u->prev;
	if (u->prev)
	    u->prev->next = u->next;
	else
	    c->voices = u->next;
	free(u);
    }

    if (!c->users) {
	if (debug)
	    log("debug: Deleting channel %s", c->name);
	if (c->topic)
	    free(c->topic);
	if (c->key)
	    free(c->key);
	if (c->bancount) {
	    int i;
	    for (i = 0; i < c->bancount; ++i)
		free(c->bans[i]);
	}
	if (c->bansize)
	    free(c->bans);
	if (c->chanops || c->voices)
	    log("channel: Memory leak freeing %s: %s%s%s %s non-NULL!",
			c->name,
			c->chanops ? "c->chanops" : "",
			c->chanops && c->voices ? " and " : "",
			c->voices ? "c->voices" : "",
			c->chanops && c->voices ? "are" : "is");
	if (c->next)
	    c->next->prev = c->prev;
	if (c->prev)
	    c->prev->next = c->next;
	else
	    chanlist = c->next;
	free(c);
    }
}

/*************************************************************************/

/* Handle a channel MODE command. */

void do_cmode(const char *source, int ac, char **av)
{
    Channel *chan;
    struct c_userlist *u;
    User *user;
    char *s, *nick;
    int add = 1;		/* 1 if adding modes, 0 if deleting */
    char *modestr = av[1];

    chan = findchan(av[0]);
    if (!chan) {
	log("channel: MODE %s for nonexistent channel %s",
                                     merge_args(ac-1, av+1), av[0]);
	return;
    }
    s = modestr;
    ac -= 2;
    av += 2;

    while (*s) {

	switch (*s++) {

	case '+':
	    add = 1; break;

	case '-':
	    add = 0; break;

	case 'i':
	    if (add)
		chan->mode |= CMODE_I;
	    else
		chan->mode &= ~CMODE_I;
	    break;

	case 'm':
	    if (add)
		chan->mode |= CMODE_M;
	    else
		chan->mode &= ~CMODE_M;
	    break;

        case 'R':
	    if (add)
                chan->mode |= CMODE_r;
	    else
                chan->mode &= ~CMODE_r;
	    break;

	case 'n':
	    if (add)
		chan->mode |= CMODE_N;
	    else
		chan->mode &= ~CMODE_N;
	    break;

	case 'p':
	    if (add)
		chan->mode |= CMODE_P;
	    else
		chan->mode &= ~CMODE_P;
	    break;

	case 's':
	    if (add)
		chan->mode |= CMODE_S;
	    else
		chan->mode &= ~CMODE_S;
	    break;

	case 't':
	    if (add)
		chan->mode |= CMODE_T;
	    else
		chan->mode &= ~CMODE_T;
	    break;

	case 'c':
	    if (add)
		chan->mode |= CMODE_C;
	    else
		chan->mode &= ~CMODE_C;
	    break;

	case 'k':
	    if (--ac < 0) {
		log("channel: MODE %s %s: missing parameter for %ck",
					chan->name, modestr, add ? '+' : '-');
		break;
	    }
	    if (chan->key) {
		free(chan->key);
		chan->key = NULL;
	    }
	    if (add)
		chan->key = sstrdup(*av++);
	    break;

	case 'l':
	    if (add) {
		if (--ac < 0) {
		    log("channel: MODE %s %s: missing parameter for +l",
							chan->name, modestr);
		    break;
		}
		chan->limit = atoi(*av++);
	    } else {
		chan->limit = 0;
	    }
	    break;

	case 'b':
	    if (--ac < 0) {
		log("channel: MODE %s %s: missing parameter for %cb",
					chan->name, modestr, add ? '+' : '-');
		break;
	    }
	    if (add) {
		if (chan->bancount >= chan->bansize) {
		    chan->bansize += 8;
		    chan->bans = srealloc(chan->bans,
					sizeof(char *) * chan->bansize);
		}
		chan->bans[chan->bancount++] = sstrdup(*av++);
	    } else {
		char **s = chan->bans;
		int i = 0;
		while (i < chan->bancount && strcmp(*s, *av) != 0) {
		    ++i; ++s;
		}
		if (i < chan->bancount) {
		    --chan->bancount;
		    if (i < chan->bancount)
			bcopy(s+1, s, sizeof(char *) * (chan->bancount-i));
		}
		++av;
	    }
	    break;

	case 'o':
	    if (--ac < 0) {
		log("channel: MODE %s %s: missing parameter for %co",
					chan->name, modestr, add ? '+' : '-');
		break;
	    }
	    nick = *av++;
	    if (add) {
		for (u = chan->chanops; u && stricmp(u->user->nick, nick) != 0;
								u = u->next)
		    ;
		if (u)
		    break;
		user = finduser(nick);
		if (!user) {
		    log("channel: MODE %s +o for nonexistent user %s",
							chan->name, nick);
		    break;
		}
		if (debug)
		    log("debug: Setting +o on %s for %s", chan->name, nick);
#ifndef SKELETON
                if (!check_valid_op(user, chan->name, !!strchr(source, '.'))) {
                       send_cmd(s_ChanServ, "MODE %s -o %s", chan->name, nick);
                       break;
                } else {
#endif
		u = smalloc(sizeof(*u));
		u->next = chan->chanops;
		u->prev = NULL;
		if (chan->chanops)
		    chan->chanops->prev = u;
		chan->chanops = u;
		u->user = user;
#ifndef SKELETON
              }
#endif
	    } else {
		for (u = chan->chanops; u && stricmp(u->user->nick, nick);
								u = u->next)
		    ;
		if (!u)
		    break;
		if (u->next)
		    u->next->prev = u->prev;
		if (u->prev)
		    u->prev->next = u->next;
		else
		    chan->chanops = u->next;
		free(u);
	    }
	    break;

	case 'v':
	    if (--ac < 0) {
		log("channel: MODE %s %s: missing parameter for %cv",
					chan->name, modestr, add ? '+' : '-');
		break;
	    }
	    nick = *av++;
	    if (add) {
		for (u = chan->voices; u && stricmp(u->user->nick, nick);
								u = u->next)
		    ;
		if (u)
		    break;
		user = finduser(nick);
		if (!user) {
		    log("channel: MODE %s +v for nonexistent user %s",
							chan->name, nick);
		    break;
		}
		if (debug)
		    log("debug: Setting +v on %s for %s", chan->name, nick);
		u = smalloc(sizeof(*u));
		u->next = chan->voices;
		u->prev = NULL;
		if (chan->voices)
		    chan->voices->prev = u;
		chan->voices = u;
		u->user = user;
	    } else {
		for (u = chan->voices; u && stricmp(u->user->nick, nick);
								u = u->next)
		    ;
		if (!u)
		    break;
		if (u->next)
		    u->next->prev = u->prev;
		if (u->prev)
		    u->prev->next = u->next;
		else
		    chan->voices = u->next;
		free(u);
	    }
	    break;

	} /* switch */

    } /* while (*s) */

#ifndef SKELETON
    /* Check modes against ChanServ mode lock */
         check_modes(chan->name);
#endif
}

/*************************************************************************/

/* Handle a TOPIC command. */

void do_topic(const char *source, int ac, char **av)
{
    Channel *c = findchan(av[0]);

    if (!c) {
	log("channel: TOPIC %s for nonexistent channel %s",
						merge_args(ac-1, av+1), av[0]);
	return;
    }
#ifndef SKELETON
    if (!strchr(source, '.')) {
        if (check_topiclock(source, av[0]))
           return;
    } else
           return;


#endif
    strscpy(c->topic_setter, av[1], sizeof(c->topic_setter));
    c->topic_time = atol(av[2]);
    if (c->topic) {
	free(c->topic);
	c->topic = NULL;
    }
    if (ac > 3 && *av[3])
	c->topic = sstrdup(av[3]);
#ifndef SKELETON
    record_topic(av[0]);
#endif
}

/*************************************************************************/

/* Does the given channel have only one user? */

/* Note:  This routine is not currently used, but is kept around in case it
 * might be handy someday. */

#if 0
int only_one_user(const char *chan)
{
    Channel *c;

    for (c = chanlist; c; c = c->next) {
	if (stricmp(c->name, chan) == 0)
	    return (c->users && !c->users->next) ? 1 : 0;
    }
    return 0;
}
#endif

int isin_chan(char *nick, const char *chan)
{
    Channel *c;
    struct c_userlist *cu;

    for (c = chanlist; c; c = c->next) {
        if (stricmp(c->name, chan) == 0) {
	    for (cu = c->users; cu; cu = c->users->next) {
		if (stricmp(cu->user->nick, nick) == 0)
		    return 1;
	    }
	    return 0;
	}
    }
    return 0;
}


/*************************************************************************/
void change_cmode (const char *who, const char *chan, const char *mode, const char *pram)
{
    char *av[3];
    av[0] = sstrdup (chan);
    av[1] = sstrdup (mode);
    av[2] = sstrdup (pram);
    {
        send_cmd (who, "MODE %s %s %s", chan, mode, pram);
        do_cmode (who, 3, av);
    }
    free (av[2]);
    free (av[1]);
    free (av[0]);
}

